@@ -229,7 +229,7 @@ def add_arguments_extended(parser: KGTKArgumentParser, parsed_shared_args: Names
                               help="If true and filtering labels by language, pick only the label matching " +
                               "the language that appears before other matches in the language list. (default=%(default)s).",
                               metavar="True/False",
-                              type=optional_bool, nargs='?', const=True, default=use_label_envar_default)
+                              type=optional_bool, nargs='?', const=True, default=False)
 
     parser.add_argument(      "--use-label-envar", dest="use_label_envar",
                               help="If true, use the KGTK_LABEL_FILE envar for the label file if no --label-file. (default=%(default)s).",
@@ -251,6 +251,11 @@ def add_arguments_extended(parser: KGTKArgumentParser, parsed_shared_args: Names
                               metavar="True/False",
                               type=optional_bool, nargs='?', const=True, default=force_input_mode_none_default)
         
+    parser.add_argument(      "--strip-language-qualifiers", dest="strip_language_qualifiers",
+                              help=h("If true, strip language qualifiers from lifted labels). (default=%(default)s)."),
+                              metavar="True/False",
+                              type=optional_bool, nargs='?', const=True, default=False)
+
     KgtkReader.add_debug_arguments(parser, expert=_expert)
     # TODO: seperate reader_options for the label file.
     KgtkReaderOptions.add_arguments(parser,
@@ -304,6 +309,7 @@ def run(input_file: KGTKFiles,
         lift_all_columns: bool = False,
         require_label_file: bool = False,
         force_input_mode_none: bool = False,
+        strip_language_qualifiers: bool = False,
 
         errors_to_stdout: bool = False,
         errors_to_stderr: bool = True,
@@ -396,6 +402,7 @@ def run(input_file: KGTKFiles,
         print("--lift-all-columns=%s" % repr(lift_all_columns), file=error_file, flush=True)
         print("--require-label-files=%s" % repr(require_label_file), file=error_file, flush=True)
         print("--force-input-mode-none=%s" % repr(force_input_mode_none), file=error_file, flush=True)
+        print("--strip-language-qualifiers=%s" % repr(strip_language_qualifiers), file=error_file, flush=True)
         input_reader_options.show(out=error_file, who="input")
         label_reader_options.show(out=error_file, who="label")
         value_options.show(out=error_file)
@@ -456,6 +463,7 @@ def run(input_file: KGTKFiles,
 
             lift_all_columns=lift_all_columns,
             force_input_mode_none=force_input_mode_none,
+            strip_language_qualifiers=strip_language_qualifiers,
 
             input_reader_options=input_reader_options,
             label_reader_options=label_reader_options,
